use std::{error, fmt, io, num::TryFromIntError};

#[derive(Debug)]
pub enum Error {
    Io(io::Error),
    IllegalChar(char),
    IntError(TryFromIntError),
    MissingPadding,
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Io(e) => write!(f, "{e}"),
            Self::IllegalChar(c) => write!(f, "Illegal Character ({c})"),
            Self::MissingPadding => write!(f, "Missing Padding"),
            Self::IntError(e) => write!(f, "Int Error: {e}"),
        }
    }
}

impl From<io::Error> for Error {
    fn from(value: io::Error) -> Self {
        Self::Io(value)
    }
}

impl From<char> for Error {
    fn from(value: char) -> Self {
        Self::IllegalChar(value)
    }
}

impl From<TryFromIntError> for Error {
    fn from(value: TryFromIntError) -> Self {
        Self::IntError(value)
    }
}

impl error::Error for Error {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::Io(e) => Some(e),
            Self::IntError(e) => Some(e),
            _ => None,
        }
    }
}
